from random import sample
import sys

# 1. Create hash table for megan specific gtdb_id -> custom_megan_id mappings
cmm_dict = {}   # custom megan mapping dict

with open("annotree_tmp/gtdb.map", "r") as file:
    for line in file:
        line = line.strip().split("\t")
        if len(line) == 5:
            cid = line[0]   # custom megan id
            gid = line[4]   # gtdb id

            if gid not in cmm_dict:
                cmm_dict[gid] = cid
            else:
                print("Duplicate in gtdb_id -> custom_megan_id mappings. This should not be.")
                exit()

# 2. Create hash table for gtdb_id -> taxonomy_id mapping
ntax_dict = {}  # node taxonomy dict

with open("annotree_tmp/node_tax.tsv", "r") as file:
    for line in file:
        gid, tid = line.strip().split("\t")
        gid = gid.split(".")[0]     # remove version numbers which are not represented in megan

        if gid not in ntax_dict:
            ntax_dict[gid] = tid
        else:
            print("Duplicated gtdb_id. This should not be.")
            exit()

# 3. Create hash table for accession -> kegg_id/pfam_id/tigrfam_id
kth_dict = {}       # kegg_top_hits dict
pth_dict = {}       # pfam_top_hits dict
tth_dict = {}       # tigrfam_top_hits dict

for col, dct in [["kegg", kth_dict], ["pfam", pth_dict], ["tigrfam", tth_dict]]:
    with open("annotree_tmp/{}_top_hits.tsv".format(col), "r") as file:
        for line in file:
            gene_id, gtdb_id, col_id = line.strip().split("\t")
            accession = "{}__{}".format(gene_id, gtdb_id)
            accession = accession.replace(".", "_")

            if accession not in dct:
                dct[accession] = [col_id]
            else:
                dct[accession].append(col_id)

# 4. Iterate over all accessions from protein_sequences tables, translate kegg ids and gtdb ids into megan specific
#    integers and build Accession -> Taxonomy -> GTDB -> KEGG mapping
with open("annotree_tmp/mapping_{}.tsv".format(sys.argv[1]), "w") as out:
    with open("annotree_tmp/accession2gtdb_id.tsv", "r") as file:
        for line in file:
            accession, gid = line.strip().split("\t")
            gid = gid.split(".")[0]     # remove version numbers which are not represented in megan

            # Taxonomy
            try:
                taxonomy = ntax_dict[gid]
            except KeyError:
                print("No taxonomy found for gtdb id. This should not be.")
                exit()

            # GTDB
            try:
                gtdb = cmm_dict[gid]
            except KeyError:
                print("No custom megan gtdb integer found for gtdb id. This should not be.")
                exit()

            # KEGG
            try:
                kegg_list = kth_dict[accession]
                kegg = sample(kegg_list, 1)[0]  # random pick of only one kegg id per accession due to megan limitations

                # the following is done to translate kegg ids into megan specific integers
                kegg = kegg[1:]  # drop "K" from id (e.g. K00086 -> 00086)
                kegg = int(kegg)  # drop leading zeros (e.g. 00086 -> 86)
            except KeyError:
                kegg = ""

            # PFAM
            try:
                pfam_list = pth_dict[accession]
                pfam = sample(pfam_list, 1)[0]  # random pick of only one pfam id per accession due to megan limitations

                # the following is done to translate pfam ids into megan specific integers
                pfam = pfam[2:]  # drop "PF" from id (e.g. PF00766 -> 00766)
                pfam = int(pfam)  # drop leading zeros (e.g. 00766 -> 766)
            except KeyError:
                pfam = ""

            # TIGRFAM
            try:
                tigrfam_list = tth_dict[accession]
                tigrfam = sample(tigrfam_list, 1)[0]  # random pick of only one tigrfam id per accession due to megan
                # limitations 

                # the following is done to translate tigrfam ids into megan specific integers
                tigrfam = tigrfam[4:]  # drop "TIGR" from id (e.g. TIGR02537 -> 02537)
                tigrfam = int(tigrfam)  # drop leading zeros (e.g. 02537 -> 2537)
            except KeyError:
                tigrfam = ""

            # Write output in format Accession|Taxonomy|GTDB|EGGNOG|INTERPRO2GO|SEED|EC|KEGG|PFAM|TIGRFAM which ..
            # .. extends table "mappings" from original megan-map-Jul2020-2-ue.db by PFAM and TIGRFAM
            out.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(accession, taxonomy, gtdb, "", "", "", "", kegg, 
                                                                        pfam, tigrfam))
